
#include "x.h"
#include "xt.h"
#include "xpex.h"
#include <math.h>
#include "book_utils.h"
#include "externs.h"
#include "defs.h"

XtAppContext  app;
Display      *dpy;
Window        window;
Drawable      drawable;
Multibuffer   buffers[2];
Pixmap        pixmap;
XWindowAttributes    xwattrs;
GC            gc;
Atom          delete_window_atom;

PEXDepthCueEntry depth_cue_entry;

PEXRenderer           renderer;
PEXRendererAttributes rattrs;

PEXStructure top_struct, sphere;

Widget toplevel, canvas;

PEXColorSpecifier colorSpec;

Boolean colorTypeIndexed, colorTypeRGB,
        colorTypeHSV, colorTypeHLS, colorTypeCIE;

PEXTableIndex        listON[4] = { 1, 2 };
/* PEXTableIndex        listOFF[4] = { 3, 4 }; */
PEXTableIndex        listOFF[4];
PEXLightEntry        lights[4];

/* The viewing parameters. */
PEXCoord         view_ref_pt = { 0.5,0.5,0.5};
PEXVector        view_up_vec = { 0,1,0};
PEXVector        view_plane_normal = { 1,1,1};
PEXCoord2D       view_window[2];
PEXNPCSubVolume  viewport;
double           view_plane, front_plane, back_plane;
PEXCoord         prp;
int              perspective;
PEXViewEntry     view;

int using_drawable = BUFFER;

main( argc, argv )
int        argc;
char    *argv[];
{
    XVisualInfo            vis_info;
    Colormap        colormap;
    XStandardColormap        cmap_info;
    PEXColorApproxEntry        capx_info;
    XSetWindowAttributes    wattrs;
    PEXExtensionInfo        *pexinfo;
    char            err[80];
    extern int PrintDefaultError();
    int                  type;
    PEXColor color;
    XColor background;

    unsigned long   pcMask[3];
    PEXPCAttributes	pcattrs;

    initialize_depth_cue_table();

    if( argc > 1 ) {
    float stof();
    depth_cue_entry.back_plane    = stof( argv[4] );
    depth_cue_entry.front_plane   = stof( argv[6] );
    depth_cue_entry.back_scaling  = stof( argv[8] );
    depth_cue_entry.front_scaling = stof( argv[10] );
    depth_cue_entry.color.type = PEXColorTypeRGB;
    SET_COLOR( stof( argv[15] ), stof( argv[16] ), stof( argv[17] ), depth_cue_entry.color.value );
    }

    /* Initialize the toolkit. */
    toplevel = XtVaAppInitialize( &app, "XDepthCueEditor",
        (XrmOptionDescList)NULL, 0,
        (int *)&argc, (String *)argv,
        (String *)NULL, NULL );

    /* See what display is being used. */
    dpy = XtDisplay( toplevel );
    XSetErrorHandler( PrintDefaultError );

    /* See if a PEX extension exists and initialize PEXlib. */
    if ( PEXInitialize( dpy, &pexinfo, 80, err ) != 0 ) {
        fprintf( stderr, "No PEX extension.\n" );
        fprintf( stderr, "%s\n", err );
        exit(1);
    }

    /* Determine the best visual to use. */
    ora_find_best_visual( dpy, &vis_info );

    if( vis_info.depth != 8 )
        ora_create_colormap( dpy, &vis_info, &cmap_info, &capx_info );
    else {
        /* Get a standard colormap for the visual. */
        if ( !ora_get_standard_colormap( dpy, &vis_info, &cmap_info ) ) {
            fprintf ( stderr, "Cannot find a standard colormap\n" );
            exit(1);
        }
        ora_set_stdcmap_approx( &vis_info, &cmap_info, &capx_info );
    }

    if( !(strcmp( pexinfo->vendor_name, "International Business Machines" ) ) )
        XtVaSetValues( toplevel,
            XtNvisual, vis_info.visual,
            XtNdepth, vis_info.depth,
            XtNcolormap, cmap_info.colormap,
            NULL );

    create_gui( vis_info.depth, pexinfo->vendor_name );
    XtRealizeWidget( toplevel );

    if( !(strcmp( pexinfo->vendor_name, "International Business Machines" ) ) )
        window = XtWindow( canvas );
    else {
        background.pixel = 0;
        background.flags = DoRed | DoGreen | DoBlue;
        window = ora_create_window( dpy, XtWindow(canvas), &vis_info,&cmap_info,
            "210x210", &background );
    }

    if( PEXUtDBConfig( dpy, window, True, &background, False, &drawable ) ) {
        fprintf( stderr, "No Double Buffering Support\n" );
        drawable = window;
    }

    /* Create the PEX renderer. */
    if ( IMMED_MODE_SPT( pexinfo ) ) {
        renderer = ora_setup_renderer( dpy, drawable, &capx_info, &rattrs );
    } else {
        fprintf( stderr, "Immediate mode not supported.\n" );
        exit(1);
    }

    pcMask[0] = pcMask[1] = pcMask[2] = 0;
    pcattrs.distinguish          = True;
    PEXSetPCAttributeMask( pcMask, PEXPCDistinguishFlag );
    pcattrs.interior_style       = PEXInteriorStyleHollow;
    PEXSetPCAttributeMask( pcMask, PEXPCInteriorStyle );
    pcattrs.bf_interior_style    = PEXInteriorStyleHollow;
    PEXSetPCAttributeMask( pcMask, PEXPCBFInteriorStyle );
    pcattrs.light_state.count    = 2;
    pcattrs.light_state.indices  = listON;
    PEXSetPCAttributeMask( pcMask, PEXPCLightState );
    pcattrs.depth_cue_index      = 1;
    PEXSetPCAttributeMask( pcMask, PEXPCDepthCueIndex );
    rattrs.pipeline_context = 
    PEXCreatePipelineContext( dpy, pcMask, &pcattrs );
    PEXChangeRenderer( dpy, renderer, PEXRAPipelineContext, &rattrs );

    colorTypeIndexed = ora_enum_type_supported( dpy, drawable,
        PEXETColorType, PEXColorTypeIndexed );
    colorTypeRGB = ora_enum_type_supported( dpy, drawable,
        PEXETColorType, PEXColorTypeRGB );
    colorTypeHSV = ora_enum_type_supported( dpy, drawable,
        PEXETColorType, PEXColorTypeHSV );
    colorTypeHLS = ora_enum_type_supported( dpy, drawable,
        PEXETColorType, PEXColorTypeHLS );
    colorTypeCIE = ora_enum_type_supported( dpy, drawable,
        PEXETColorType, PEXColorTypeCIE );

    build_sphere();

    top_struct = PEXCreateStructure( dpy );

    PEXSetTableEntries( dpy, rattrs.depth_cue_table, 1, 1,
        PEXLUTDepthCue, &depth_cue_entry );

#if( 0 )    /* moved into a pc */
    PEXSetFacetDistinguishFlag( dpy, top_struct, PEXOCStore, True );

    PEXSetLightSourceState( dpy, top_struct, PEXOCStore, 2,listON, 0,listOFF );
#endif		/*  */

#if( 0 )    /* moved into a pc */
    PEXSetDepthCueIndex( dpy, top_struct, PEXOCStore, 1 );
#endif		/*  */
    PEXSetViewIndex( dpy, top_struct, PEXOCStore, 1 );
    PEXLabel( dpy, top_struct, PEXOCStore, OBJECT_LABEL );
    PEXExecuteStructure( dpy, top_struct, PEXOCStore, sphere );

    initialize_view();
    initialize_light_table();

    for (;;) {
        XEvent event;
        XtAppNextEvent(app, &event);
        if( (event.type == Expose) && 
            (event.xexpose.window == window) &&
            (event.xexpose.count == 0) )
            redraw();
        else
            XtDispatchEvent(&event);
    }
}


float stof( buf )
char *buf;
{
    float f;
    sscanf( buf, "%f", &f );
    return f;
}


void
redraw()
{
    /* Redraw the window, after first erasing (clearing) it. */
    PEXRenderNetwork( dpy, drawable, renderer, top_struct );
    drawable = PEXUtDBSwap( dpy, window, False );
}

#define NORMALIZE( _v ) \
{ \
    double    mag; \
\
    mag = 1.0 / sqrt((_v).x * (_v).x \
          + (_v).y * (_v).y + (_v).z * (_v).z); \
    (_v).x *= mag, (_v).y *= mag, (_v).z *= mag; \
}

static void
initialize_light_table()
{
    lights[AMBIENT].type = PEXLightAmbient;
    lights[AMBIENT].color.type = PEXColorTypeRGB;
    SET_COLOR( 1, 1, 1, lights[AMBIENT].color.value );
    PEXSetTableEntries( dpy, rattrs.light_table, 1, 1,
        PEXLUTLight, &lights[AMBIENT] );

    lights[DIRECTIONAL].type = PEXLightWCVector;
    lights[DIRECTIONAL].direction.x = -1;
    lights[DIRECTIONAL].direction.y =  0;
    lights[DIRECTIONAL].direction.z =  0;
    NORMALIZE(lights[DIRECTIONAL].direction);
    lights[DIRECTIONAL].color.type = PEXColorTypeRGB;
    SET_COLOR( 1, 1, 1, lights[DIRECTIONAL].color.value );
    PEXSetTableEntries( dpy, rattrs.light_table, 2, 1,
        PEXLUTLight, &lights[DIRECTIONAL] );
#if( 0 ) 
    lights[POSITIONAL].type = PEXLightWCPoint;
    lights[POSITIONAL].point.x = 0.5;
    lights[POSITIONAL].point.y = 2.0;
    lights[POSITIONAL].point.z = 0.5;
    lights[POSITIONAL].attenuation1 = 0;
    lights[POSITIONAL].attenuation2 = 1;
    lights[POSITIONAL].color.type = PEXColorTypeRGB;
    SET_COLOR( 1, 1, 1, lights[POSITIONAL].color.value );
    PEXSetTableEntries( dpy, rattrs.light_table, 3, 1,
        PEXLUTLight, &lights[POSITIONAL] );

    lights[SPOT].type = PEXLightWCSpot;
    lights[SPOT].point.x = 0.5;
    lights[SPOT].point.y = 2.0;
    lights[SPOT].point.z = 0.5;
    lights[SPOT].direction.x = 0;
    lights[SPOT].direction.y = -1;
    lights[SPOT].direction.z = 0;
    NORMALIZE(lights[SPOT].direction);
    lights[SPOT].attenuation1 = 0;
    lights[SPOT].attenuation2 = 1;
    lights[SPOT].concentration = 2;
    lights[SPOT].spread_angle = M_PI/2;
    lights[SPOT].color.type = PEXColorTypeRGB;
    SET_COLOR( 1, 1, 1, lights[SPOT].color.value );
    PEXSetTableEntries( dpy, rattrs.light_table, 4, 1,
        PEXLUTLight, &lights[SPOT] );
#endif		/*  */
}

void
initialize_depth_cue_table()
{
    depth_cue_entry.mode = PEXOn;
    depth_cue_entry.back_plane    = 0.1;
    depth_cue_entry.front_plane   = 0.8;
    depth_cue_entry.back_scaling  = 0.5;
    depth_cue_entry.front_scaling = 1.0;
    depth_cue_entry.color.type = PEXColorTypeRGB;
    SET_COLOR( 0, 0, 0, depth_cue_entry.color.value );
}

void
initialize_view()
{
    int error;

    /* Compute the view orientation matrix. */
    error = PEXViewOrientationMatrix( &view_ref_pt, &view_plane_normal,
        &view_up_vec, view.orientation );

    /* Set the view mapping parameters. */
    perspective = True;
    prp.x = 0; 
    prp.y = 0; 
    prp.z = 10.0;
    view_window[0].x = -1.5; 
    view_window[1].x =  1.5;
    view_window[0].y = -1.5; 
    view_window[1].y =  1.5;
    back_plane = -2.0;
    front_plane = 2.0; 
    view_plane = 0.0;
    viewport.min.x = 0.0;    viewport.max.x = 1.0;
    viewport.min.y = 0.0;    viewport.max.y = 1.0;
    viewport.min.z = 0.0;    viewport.max.z = 1.0;
    error = PEXViewMappingMatrix( view_window, &viewport, perspective,
        &prp, view_plane, back_plane,
        front_plane, view.mapping );

    /* Set the view representation. */
    view.clip_flags = PEXClippingAll;
    view.clip_limits = viewport;
    PEXSetTableEntries( dpy, rattrs.view_table, 1, 1, PEXLUTView, &view );
}


#define NUM_LONG_FACETS   16
#define NUM_LONG_POINTS   (NUM_LONG_FACETS + 1)
#define NUM_LAT_FACETS    8
#define NUM_LAT_POINTS    (NUM_LAT_FACETS + 1)
#define RADIUS            1.00

void
build_sphere()
{
    PEXVertexNormal     vertex_data[NUM_LONG_POINTS * NUM_LAT_POINTS];
    PEXArrayOfVertex    vdata;
    PEXArrayOfFacetData facet_data;
    PEXVector           facet_normals[NUM_LONG_FACETS * NUM_LAT_FACETS];
    unsigned int        facet_data_flag, vertex_data_flag;
    double              theta = 0, phi = 0;
    register int        i, j, index;

    facet_data_flag = PEXGANormal;
    vertex_data_flag = PEXGANormal;

    for ( i = 0; i < NUM_LAT_POINTS; i++ ) {
        for ( j = 0; j < NUM_LONG_FACETS; j++ ) {
            index = i * NUM_LONG_POINTS + j;
            vertex_data[index].point.x = RADIUS * (1 + sin(phi) * cos(theta));
            vertex_data[index].point.y = RADIUS * (1 + cos(phi));
            vertex_data[index].point.z = RADIUS * (1 + sin(phi) * sin(theta));
            vertex_data[index].normal.x = sin(phi) * cos(theta);
            vertex_data[index].normal.y = cos(phi);
            vertex_data[index].normal.z = sin(phi) * sin(theta);
            NORMALIZE(vertex_data[index].normal);
            theta += (2 * M_PI) / NUM_LONG_FACETS;
        }
        phi += M_PI / NUM_LAT_FACETS;

        /* Close the circle. */
        index = i * NUM_LONG_POINTS + NUM_LONG_FACETS;
        vertex_data[index].point = vertex_data[index - NUM_LONG_FACETS].point;
        vertex_data[index].normal = vertex_data[index - NUM_LONG_FACETS].normal;
    }

    theta = M_PI / NUM_LONG_FACETS;
    phi = 0.5 * M_PI / NUM_LAT_FACETS;
    index = 0;
    for ( i = 0; i < NUM_LAT_FACETS; i++ ) {
        for ( j = 0; j < NUM_LONG_FACETS; j++ ) {
            facet_normals[index].x = sin(phi) * cos(theta);
            facet_normals[index].y = cos(phi);
            facet_normals[index].z = sin(phi) * sin(theta);
            ++index;
            theta += (2 * M_PI) / NUM_LONG_FACETS;
        }
        phi += M_PI / NUM_LAT_FACETS;
    }

    sphere = PEXCreateStructure( dpy );
    vdata.normal = vertex_data;
    facet_data.normal = facet_normals;
    PEXQuadrilateralMesh( dpy, sphere, PEXOCStore, PEXShapeUnknown,
        facet_data_flag, vertex_data_flag,
        PEXColorTypeRGB, facet_data,
        NUM_LONG_POINTS, NUM_LAT_POINTS, vdata );
}
