
#include "x.h"
#include "xt.h"
#ifndef USE_ATHENA
#include "xm.h"
#endif
#include "xpex.h"
#include "off.h"
#include "defs.h"
#include "externs.h"
#include "dialog.h"
#include "menus.h"
#include "global.h"
#include "resources.h"          /* fallback resources needed for
                                   reasonable standalone operation */
#include <X11/Xmu/Editres.h>

#ifdef XPM
#include <X11/xpm.h>
#endif

#include "unix.h"

/* pexview Icon */
#include "pixmaps/geom.xbm"
#include "pixmaps/quit.xbm"

XtAppContext app_context;

Display *display;

Widget toplevel, w_canvas, w_visual,
       w_control, w_control_visual, w_control_material_predef,
       w_control_video, w_control_print,
       w_material_predef, w_video, w_hardcopy,
       w_wireframe, w_cull, w_edge, w_shadow, w_vanish,
       w_xwd_text, w_filter_text, w_print_text,
       w_about, w_reset, w_color, w_material, w_light, w_view, w_depthCue,
       w_flybycr, w_flybyex, w_animcr, w_animex, w_morph, w_spin, w_melt;

String XwdCommand, FilterCommand, PrintCommand;
Pixmap geom_icon, quitPixmap, quitBitmap, groundPixmap, groundBitmap;
Movie frame, head, copy_head;

Atom wm_delete_window;

char *pattern;
extern Dialog dialog_s, dialog_e, dialog_g, dialog_a;

/* program line options */
static XrmOptionDescRec options[] =
{
    {"-xwdcommand", "xwdcommand", XrmoptionSepArg, NULL},
    {"-filtercommand", "filtercommand", XrmoptionSepArg, NULL},
    {"-printcommand", "printcommand", XrmoptionSepArg, NULL},
};


/* new resources defined */
static XtResource resources[] =
{
    { "xwdcommand", "XwdCommand",
    XtRString, sizeof(char *), (Cardinal)&XwdCommand, XtRString, "xwd -frame"
    },
    { "filtercommand", "FilterCommand",
    XtRString, sizeof(char *), (Cardinal)&FilterCommand, XtRString, "xwd2ps"
    },
    { "printcommand", "PrintCommand",
    XtRString, sizeof(char *), (Cardinal)&PrintCommand, XtRString, "lpr -Pqms2"
    },
};


void quit (w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    extern OFFObjDesc *Obj;
    extern int using_drawable;
    extern Window window;

    if (event->type == ClientMessage &&
        event->xclient.data.l[0] != wm_delete_window) {
        XBell (XtDisplay(w), 0);
        return;
    }
    if( Obj ) OFFFreeObj( Obj );
    if( using_drawable == BUFFER && window != (Window)NULL )
        XmbufDestroyBuffers( display, window );
    XtDestroyApplicationContext( app_context );
    exit (0);
}


#if( 0 ) 
void client_msg( shell, parm, event, flag )
Widget shell;
caddr_t * parm;
XEvent * event;
int * flag;
{
    if( event->type == ClientMessage ) {
        XClientMessageEvent * cm = (XClientMessageEvent *) event;
        Atom atom;

        atom = XmInternAtom( XtDisplay(toplevel), "_MOTIF_WM_OFFSET", FALSE );

        if ( cm->data.l[0] == atom )
            fprintf( stderr, "Decoration offsets: %dx%d\n",
                cm->data.l[1], cm->data.l[2]);
        else {
            char * cp = XGetAtomName( XtDisplay(toplevel), cm->data.l[0] );
            fprintf( stderr, "Some other message, atom is %s\n", cp );
            XFree( cp );
        }
    }
}
#endif		/*  */


Boolean CreateDialogs( client_data )
XtPointer client_data;
{
    static int num = 0;

    num++;

    switch( num ) {
        case 1:
            if( dialog_g == (Dialog)NULL )
                dialog_g = CreateFDialog( toplevel, "geom", Okay | Cancel );
            return False;
        case 2:
            if( dialog_a == (Dialog)NULL )
                dialog_a = CreateFDialog( toplevel, "anim", Okay | Cancel );
            return False;
        case 3:
            if( dialog_e == (Dialog)NULL )
                dialog_e = CreateDialog( toplevel, "error", Okay );
            return False;
        case 4:
            if( dialog_s == (Dialog)NULL )
                dialog_s = CreateDialog( toplevel, "save", Yes | No  );
            return False;
    }    
    return True;
}


void initXtAppTop( argc, argv )
unsigned int argc;
char **argv;
{
    void _XEditResCheckMessages();
    extern int PrintDefaultError();
    int depth;

    extern Boolean createVideoPanel();
    extern Boolean createPrintPanel();

    argv[0] = "View Obj";
    argc    = 1;
    toplevel = XtAppInitialize( &app_context, "ViewObject",
        options, XtNumber( options ),
        (int *)&argc, argv,
        fallback_resources,
        NULL, 0 );

    XtAddEventHandler( toplevel, (EventMask)0, True,
        _XEditResCheckMessages, NULL );
#if( 0 ) 
    XtAddEventHandler( toplevel, (EventMask)0, True,
        (XtEventHandler)client_msg, NULL );
#endif
    XtGetApplicationResources( toplevel, (XtPointer) NULL,
        resources, XtNumber( resources ),
        NULL, 0 );

    display = XtDisplay( toplevel );
    XSetErrorHandler( PrintDefaultError );

#ifndef USE_ATHENA
    XtVaGetValues( toplevel, XmNdepth, &depth, NULL );
#else
    XtVaGetValues( toplevel, XtNdepth, &depth, NULL );
#endif

    geom_icon = XCreatePixmapFromBitmapData( display,
        DefaultRootWindow( display ),
        geom_bits, geom_width, geom_height,
        WhitePixel( display, DefaultScreen(display) ),
        BlackPixel( display, DefaultScreen(display) ), depth );

    XtRealizeWidget( toplevel );

    pattern = (char *)malloc(5);
    XtAppAddWorkProc( app_context, CreateDialogs, (XtPointer)NULL );
    XtAppAddWorkProc( app_context, createVideoPanel, (XtPointer)NULL );
    XtAppAddWorkProc( app_context, createPrintPanel, (XtPointer)NULL );

#ifdef XDEBUG
    XSynchronize( display, True );
#endif
}


int
MakeMenuButton( Bar, Pane, Title, ResName, Mnwm )
Widget Bar;
Widget Pane;
char *Title;
char *ResName;
char *Mnwm;
{
#ifndef USE_ATHENA
    Widget Syscascade;
    Arg args[3];

    XtSetArg( args[0], XmNsubMenuId, Pane );
    XtSetArg( args[1], XmNlabelString, XmStringCreateSimple(Title) );
    XtSetArg( args[2], XmNmnemonic, Mnwm );
    Syscascade = XmCreateCascadeButton( Bar, ResName, args, 3 );
    XtManageChild( Syscascade );
#endif
    return 0;
}


XtActionProc
changeENV( w, client_data, call_data )
Widget w;
XtPointer client_data;
XtPointer call_data;
{
    extern char **environment;
    char *objs = (char *)client_data;
    int i=0;

    while( environment[i] != (char *)NULL ) {
        if( !strncmp( environment[i], "OBJ_LIST", 8 ) ) {
            environment[i][strlen(environment[i])-1] = objs[3];
            environment[i][strlen(environment[i])-2] = objs[2];
            environment[i][strlen(environment[i])-3] = objs[1];
            environment[i][strlen(environment[i])-4] = objs[0];
            return;
        }
        i++;
    }
}


XtActionProc
loadOFFObject( w, client_data, call_data )
Widget w;
XtPointer client_data;
XtPointer *call_data;
{
    char *filename;
    extern char       *FileName;
    extern OFFObjDesc *Obj;
    extern Dialog dialog_g;
    extern char * pattern;
    extern Window window;
    Position x, y;
    static int times = 0;

    quitOFFObject( w );

    FileName = (char *)malloc(10);
    FileName = "  ";
    RetryLoad:
    filename = (char *)malloc(100);
    filename = XtNewString( (char *)getenv("OBJ_PATH") );
    strcpy( pattern, "*geom" );
    if( PopupFDialog( dialog_g, "Selection", filename, &FileName ) == Okay ) {
        int start, last;
        int l;
        if( ( start = strbpos( FileName, '/' ) ) > 0 ) {
            int i;
            last = strlen( FileName );
            for( i=0; i<last-start; i++ )
                FileName[i] = FileName[start+i+1];
            FileName[last-start] = '\0';
        }
        l = strlen( FileName );
        if( ((FileName[l-1] != 'f') || (FileName[l-2] != 'f') || 
             (FileName[l-3] != 'o') || (FileName[l-4] != 'a'))
            &&
            ((FileName[l-1] != 'm') || (FileName[l-2] != 'o') || 
             (FileName[l-3] != 'e') || (FileName[l-4] != 'g')) ) {
            warning( XtParent( w ),
                "Extension \".geom\" OR \".bgeom\" is missing\0" );
            goto RetryLoad;
        }
        processFile( FileName );

        if( !times ) {
        XtSetSensitive( w_reset, True );
        XtSetSensitive( w_visual, True );
        XtSetSensitive( w_wireframe, True );
        XtSetSensitive( w_cull, True );
        XtSetSensitive( w_edge, True );
        XtSetSensitive( w_shadow, True );
        XtSetSensitive( w_morph, True );
        XtSetSensitive( w_melt, True );
        XtSetSensitive( w_vanish, True );
        XtSetSensitive( w_color, True );
        XtSetSensitive( w_material, True );
        XtSetSensitive( w_view, True );
        XtSetSensitive( w_depthCue, True );
        XtSetSensitive( w_light, True );
        XtSetSensitive( w_material_predef, True );
        XtSetSensitive( w_flybycr, True );
        XtSetSensitive( w_flybyex, True );
        XtSetSensitive( w_animcr, True );
        XtSetSensitive( w_animex, True );
        XtSetSensitive( w_video, True );
        XtSetSensitive( w_hardcopy, True );
        times = 1;
        XFlush( display );
        }

        /* createControlPanel(); */
        if( (Obj = (OFFObjDesc *)malloc( sizeof(OFFObjDesc) )) ==
            (OFFObjDesc *)0 ) {
            fprintf( stderr, "Cant allocate OFFObjDesc\n" );
            exit(1);
        }
        OFFReadObj( Obj );
        initPex( Obj );
    }
}


XtActionProc
saveOFFObject( w, client_data, call_data )
Widget w;
XtPointer client_data;
XtPointer *call_data;
{
    char   message[80];
    String filename;
    extern Dialog dialog_s;
    extern char *FileName;
    extern int smth_changed;

    if( smth_changed ) {
        RetryQuit:
        filename = XtNewString( FileName );
        sprintf( message, "Save file \"%s\" before quitting?", filename );
        if( PopupDialog( dialog_s, message, filename, &filename ) == Yes ) {
            int l = strlen(filename);
            if( (filename[l-1] != 'f') ||
                (filename[l-2] != 'f') ||
                (filename[l-3] != 'o') ||
                (filename[l-4] != 'a') ) {
                warning( XtParent( w ),
                    "Extension \".aoff\" is missing \0" );
                goto RetryQuit;
            }
            if( ! WriteAOFF( filename ) )
            {
                sprintf( message, "Can't write file: %s", filename );
                if( PopupDialog( dialog_e, message, NULL, NULL ) == Retry )
                    goto RetryQuit;
            }
        }
    }
    else
        warning( XtParent( w ), "Nothing has changed \0" );
    smth_changed = 0;
}


XtActionProc
quitOFFObject( w )
Widget w;
{
    extern int object_defined;
    extern Widget w_dialog;
    extern int    no_darea;
    extern int smth_changed;
    Movie curr;
    extern OFFObjDesc *Obj;
    extern char *FileName;
    extern Window window;
    extern Pixmap pixmap;
    extern XStandardColormap *cmap_info;
    extern int using_drawable;
    extern PEXRenderer renderer;
    extern PEXRendererAttributes rattrs;
    extern PEXMatrix init_matrix, local_matrix, shad_local_matrix;
    extern PEXStructure top_struct, object, object_shadow, axis, plane;
    extern Facet  *facets;
    extern Vertex *vertices;
    extern Vertex *morph_vertices;
    extern PEXColorRGB *facet_colors;
    extern PEXColorRGB *vertex_colors;
    extern int vertex_order;
    extern int shape_hint, ignore_edges, contour_hint, color_type;
    extern unsigned int facet_attributes, vertex_attributes;

    while( head != (Movie)NULL ) {
        curr = head;
        head = head->next;
#ifdef PIX_FRAME
        XFreePixmap( display, curr->frame );
#else
        free( (char *)curr->frame->data );
#endif
        free( (char *)curr );
    }
    head = (Movie)NULL;

    if( smth_changed ) {
        char message[80];
        String filename;
        extern Dialog dialog_s, dialog_e;

        RetryQuit:
        filename = XtNewString( FileName );
        sprintf( message, "Save file \"%s\" before quitting?", filename );
        if( PopupDialog( dialog_s, message, filename, &filename ) == Yes ) {
            int l = strlen(filename);
            if( (filename[l-1] != 'f') ||
                (filename[l-2] != 'f') ||
                (filename[l-3] != 'o') ||
                (filename[l-4] != 'a') ) {
                warning( XtParent( XtParent( w ) ),
                    "Extension \".aoff\" is missing \0" );
                goto RetryQuit;
            }
            if( ! WriteAOFF( filename ) )
            {
                sprintf( message, "Can't write file: %s", filename );
                if( PopupDialog( dialog_e, message, NULL, NULL ) == Retry )
                    goto RetryQuit;
            }
        }
    }
    smth_changed = 0;

    if( Obj ) {
        OFFFreeObj( Obj );
        Obj = (OFFObjDesc *)NULL;
    }

    if( pixmap != (Pixmap)NULL ) {
        XFreePixmap( display, pixmap );
        pixmap = (Pixmap)NULL;
    }

    if( cmap_info->colormap != (Colormap)NULL ) {
        XFreeColormap( display, cmap_info->colormap );
        cmap_info->colormap = (Colormap)NULL;
    }

    if( !no_darea ) {
        if( w_dialog != (Widget)NULL ) {
            XtPopdown( w_dialog );
            XtDestroyWidget( w_dialog );
        }
    }
    else {
        if( window != (Window)NULL )
            XDestroyWindow( display, window );
    }
    window = (Window)NULL;

    if( top_struct != (PEXStructure)NULL ) {
        PEXDestroyStructures( display, 1, &top_struct );
        top_struct = (PEXStructure)NULL;
    }
    if( object != (PEXStructure)NULL ) {
        PEXDestroyStructures( display, 1, &object );
        object = (PEXStructure)NULL;
    }
    if( object_shadow != (PEXStructure)NULL ) {
        PEXDestroyStructures( display, 1, &object_shadow );
        object_shadow = (PEXStructure)NULL;
    }
    if( axis != (PEXStructure)NULL ) {
        PEXDestroyStructures( display, 1, &axis );
        axis = (PEXStructure)NULL;
    }
    if( plane != (PEXStructure)NULL ) {
        PEXDestroyStructures( display, 1, &plane );
        plane = (PEXStructure)NULL;
    }
    if( rattrs.marker_bundle != (PEXLookupTable)NULL ) {
        PEXFreeLookupTable( display, rattrs.marker_bundle );
        rattrs.marker_bundle = (PEXLookupTable)NULL;
    }
    if( rattrs.text_bundle != (PEXLookupTable)NULL ) {
        PEXFreeLookupTable( display, rattrs.text_bundle );
        rattrs.text_bundle = (PEXLookupTable)NULL;
    }
    if( rattrs.line_bundle != (PEXLookupTable)NULL ) {
        PEXFreeLookupTable( display, rattrs.line_bundle );
        rattrs.line_bundle = (PEXLookupTable)NULL;
    }
    if( rattrs.interior_bundle != (PEXLookupTable)NULL ) {
        PEXFreeLookupTable( display, rattrs.interior_bundle );
        rattrs.interior_bundle = (PEXLookupTable)NULL;
    }
    if( rattrs.edge_bundle != (PEXLookupTable)NULL ) {
        PEXFreeLookupTable( display, rattrs.edge_bundle );
        rattrs.edge_bundle = (PEXLookupTable)NULL;
    }
    if( rattrs.view_table != (PEXLookupTable)NULL ) {
        PEXFreeLookupTable( display, rattrs.view_table );
        rattrs.view_table = (PEXLookupTable)NULL;
    }
    if( rattrs.color_table != (PEXLookupTable)NULL ) {
        PEXFreeLookupTable( display, rattrs.color_table );
        rattrs.color_table = (PEXLookupTable)NULL;
    }
    if( rattrs.depth_cue_table != (PEXLookupTable)NULL ) {
        PEXFreeLookupTable( display, rattrs.depth_cue_table );
        rattrs.depth_cue_table = (PEXLookupTable)NULL;
    }
    if( rattrs.light_table != (PEXLookupTable)NULL ) {
        PEXFreeLookupTable( display, rattrs.light_table );
        rattrs.light_table = (PEXLookupTable)NULL;
    }
    if( rattrs.color_approx_table != (PEXLookupTable)NULL ) {
        PEXFreeLookupTable( display, rattrs.color_approx_table );
        rattrs.color_approx_table = (PEXLookupTable)NULL;
    }
    if( rattrs.text_font_table != (PEXLookupTable)NULL ) {
        PEXFreeLookupTable( display, rattrs.text_font_table );
        rattrs.text_font_table = (PEXLookupTable)NULL;
    }
    if( renderer != (PEXRenderer)NULL ) {
        PEXFreeRendererAttributes( &rattrs );
        PEXFreeRenderer( display, renderer );
        renderer = (PEXRenderer)NULL;
    }
    PEXIdentityMatrix( init_matrix );
    PEXIdentityMatrix( local_matrix );
    PEXIdentityMatrix( shad_local_matrix );
    if( facets != (Facet *)NULL ) {
        int i;
        extern unsigned int nfacets;
        for( i=0; i<nfacets; i++ )
            free( facets[i].edges );
        free( facets );
        facets = (Facet *)NULL;
    }
    if( vertices != (Vertex *)NULL ) {
        free( vertices );
        vertices = (Vertex *)NULL;
    }
    if( morph_vertices != (Vertex *)NULL ) {
        free( morph_vertices );
        morph_vertices = (Vertex *)NULL;
    }
    if( facet_colors != (PEXColorRGB *)NULL ) {
        free( facet_colors );
        facet_colors = (PEXColorRGB *)NULL;
    }
    if( vertex_colors != (PEXColorRGB *)NULL ) {
        free( vertex_colors );
        vertex_colors = (PEXColorRGB *)NULL;
    }
    vertex_order = COUNTERCLOCKWISE;
    shape_hint = ignore_edges = contour_hint = color_type = 0;
    facet_attributes = vertex_attributes = 0;
}



#ifdef XPM
void errorMessageXPM(ErrorStatus, tag)
int ErrorStatus;
char *tag;
{
    char *error = (char *)NULL;
    char *warning = (char *)NULL;

    switch (ErrorStatus) {
    case XpmSuccess:
        return;
    case XpmColorError:
        warning = "Could not parse or alloc requested color";
        break;
    case XpmOpenFailed:
        error = "Cannot open file";
        break;
    case XpmFileInvalid:
        error = "Invalid XPM file";
        break;
    case XpmNoMemory:
        error = "Not enough memory";
        break;
    case XpmColorFailed:
        error = "Failed to parse or alloc some color";
        break;
    }

    if (warning)
        printf("%s Xpm Warning: %s.\n", tag, warning);

    if (error) {
        printf("%s Xpm Error: %s.\n", tag, error);
/*      exit( -1 ); */
    }
}
#endif

void errorMessageXBM(ErrorStatus, tag)
int ErrorStatus;
char *tag;
{
    char *error = (char *)NULL;
    char *warning = (char *)NULL;

    switch (ErrorStatus) {
    case BitmapSuccess:
        return;
    case BitmapOpenFailed:
        error = "Cannot open file";
        break;
    case BitmapFileInvalid:
        error = "Invalid XBM file";
        break;
    case BitmapNoMemory:
        error = "Not enough memory";
        break;
    }

    if (warning)
        printf("%s Xpm Warning: %s.\n", tag, warning);

    if (error) {
        printf("%s Xpm Error: %s.\n", tag, error);
/*      exit( -1 ); */
    }
}


void createVisualPanel()
{
    Position x, y;

    extern XtActionProc CanvasVisual(), CanvasVisualQuit();

    Widget w_vis, w_list, w_label, w_quit;

    char errorbuf[40], *class = (char *)NULL;
    extern XVisualInfo *vis_info;
    XVisualInfo viproto;        /* fill in for getting info */
    XVisualInfo *vip;           /* retured info */
    int nvi = 0;

    String item[20];
#ifndef USE_ATHENA
    XmStringTable itemList, labelList;
#else
    String *itemList, *labelList;
#endif
    int i, itemCount;

    itemCount = 1;
    switch( vis_info->class ) {
    case StaticGray:    class = "StaticGray "; break;
    case GrayScale:     class = "GrayScale  "; break;
    case StaticColor:   class = "StaticColor"; break;
    case PseudoColor:   class = "PseudoColor"; break;
    case TrueColor:     class = "TrueColor  "; break;
    case DirectColor:   class = "DirectColor"; break;
    default:
    sprintf (errorbuf, "unknown class %d", vip->class);
    class = errorbuf;
    break;
    }
    if( (item[0] = (char *)malloc(80)) == (char *)0 ) {
        fprintf( stderr, "Cant allocate char\n" );
        exit(1);
    }
    sprintf( item[0], "visual id: 0x%lx, depth: %d, class: %s",
        vis_info->visualid, vis_info->depth, class );

    viproto.screen = DefaultScreen( display );
    vip = XGetVisualInfo( display, VisualScreenMask, &viproto, &nvi );
    itemCount += nvi;
    for( i = 1; i <= itemCount; i++ ) {
        switch( vip[i-1].class ) {
        case StaticGray:    class = "StaticGray "; break;
        case GrayScale:     class = "GrayScale  "; break;
        case StaticColor:   class = "StaticColor"; break;
        case PseudoColor:   class = "PseudoColor"; break;
        case TrueColor:     class = "TrueColor  "; break;
        case DirectColor:   class = "DirectColor"; break;
        default:
        sprintf (errorbuf, "unknown class %d", vip->class);
        class = errorbuf;
        break;
        }
        if( (item[i] = (char *)malloc(80)) == (char *)0 ) {
            fprintf( stderr, "Cant allocate char\n" );
            exit(1);
        }
        sprintf( item[i], "visual id: 0x%lx, depth: %d, class: %s",
            vip[i-1].visualid, vip[i-1].depth, class );
    }
    if( vip ) XFree( (char *)vip );

#ifndef USE_ATHENA
    if( (itemList = (XmStringTable)malloc( itemCount * sizeof(XmString *) )) ==
        (XmString *)0 ) {
        fprintf( stderr, "Cant allocate XmString\n" );
        exit(1);
    }
    for( i = 0; i < itemCount; i++ )
        itemList[i] = XmStringCreateSimple( item[i] );

    if( (labelList = (XmStringTable)malloc( sizeof(XmString *) )) ==
        (XmString *)0 ) {
        fprintf( stderr, "Cant allocate XmString\n" );
        exit(1);
    }
    labelList[0] = XmStringCreateSimple( "<-- current" );
#else
    if( (itemList = (String *)malloc( itemCount * sizeof(String *) )) ==
        (String *)0 ) {
        fprintf( stderr, "Cant allocate String\n" );
        exit(1);
    }
    for( i = 0; i < itemCount; i++ )
        itemList[i] = item[i];

    if( (labelList = (String *)malloc( sizeof(String *) )) ==
        (String *)0 ) {
        fprintf( stderr, "Cant allocate String\n" );
        exit(1);
    }
    labelList[0] = "<-- current";
#endif

    map_dialog( w_visual, &x, &y );
    w_control_visual = XtVaCreatePopupShell( "Select Visual",
        transientShellWidgetClass,
        w_visual,
        XtNx, x, XtNy, y,
        NULL );

    w_vis = XtVaCreateManagedWidget( "w_vis",
#ifndef USE_ATHENA
        xmFormWidgetClass,
        w_control_visual,
#ifdef XPM
        XmNbackgroundPixmap, groundPixmap,
#endif
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 5,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 5,
        XmNwidth, (strlen(item[0])+1) * 10,
        XmNheight, 40 + itemCount * 25,
#else
        formWidgetClass,
        w_control_visual,
#ifdef XPM
        XtNbackgroundPixmap, groundPixmap,
#endif
        XtNhorizDistance, 5,
        XtNvertDistance, 5,
        XtNwidth, (strlen(item[0])+1) * 10,
        XtNheight, 40 + itemCount * 25,
#endif
        NULL );

    w_list = XtVaCreateManagedWidget( "w_list",
#ifndef USE_ATHENA
        xmListWidgetClass,
        w_vis,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 10,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 10,
        XmNlistMarginWidth, 5,
        XmNlistMarginHeight, 5,
        XmNselectionPolicy, XmSINGLE_SELECT,
        XmNvisibleItemCount, itemCount,
        XmNitemCount, itemCount,
        XmNitems, itemList,
#else
        listWidgetClass,
        w_vis,
        XtNhorizDistance, 10,
        XtNvertDistance, 10,
        XtNdefaultColumns, 1,
        XtNnumberStrings, itemCount,
        XtNlist, itemList,
#endif
        NULL );
    XtAddCallback( w_list,
#ifndef USE_ATHENA
        XmNsingleSelectionCallback, (XtCallbackProc)CanvasVisual,
#else
        XtNcallback, (XtCallbackProc)CanvasVisual,
#endif
        (XtPointer)NULL );

#ifndef USE_ATHENA
    for( i = 0; i < itemCount; i++ )
        XmStringFree( itemList[i] );
    XtFree( (char *)itemList );
#endif

    w_label = XtVaCreateManagedWidget( "w_label",
#ifndef USE_ATHENA
        xmListWidgetClass,
        w_vis,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 10,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_list,
        XmNleftOffset, 10,
        XmNlistMarginWidth, 5,
        XmNlistMarginHeight, 3,
        XmNselectionPolicy, XmSINGLE_SELECT,
        XmNvisibleItemCount, 1,
        XmNitemCount, 1,
        XmNitems, labelList,
#else
        listWidgetClass,
        w_vis,
        XtNfromHoriz, w_list,
        XtNhorizDistance, 10,
        XtNvertDistance, 10,
        XtNdefaultColumns, 1,
        XtNnumberStrings, 1,
        XtNlist, labelList,
#endif
        NULL );

    w_quit = XtVaCreateManagedWidget( "w_quit",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass,
        w_vis,
        XmNlabelString, XmStringCreateSimple( "Quit" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_list,
        XmNtopOffset, 10,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 10,
        XmNwidth, strlen("Quit") * 10,
        XmNheight, strlen("Quit") * 10,
#else
        commandWidgetClass,
        w_vis,
        XtNlabel, "Quit",
        XtNhorizDistance, 10,
        XtNfromVert, w_list,
        XtNvertDistance, 10,
        XtNwidth, strlen("Quit") * 10,
        XtNheight, strlen("Quit") * 10,
#endif
        NULL );
    XtAddCallback( w_quit,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasVisualQuit,
#else
        XtNcallback, (XtCallbackProc)CanvasVisualQuit,
#endif
        (XtPointer)NULL );

    XtRealizeWidget( w_control_visual );
    XtPopup( w_control_visual, XtGrabNone );

#ifndef USE_ATHENA
    wm_delete_window = XmInternAtom( display, "WM_DELETE_WINDOW", False );
    XmAddWMProtocolCallback( w_control_visual, wm_delete_window,
        (XtCallbackProc)quit, NULL );
#endif
}


void createMaterialPredefPanel()
{
    extern void create_menus();
    extern XtActionProc CanvasMaterialPredef(), CanvasMaterialPredefQuit();

    static menu_struct MetalData[] = {
        { "brass",          CanvasMaterialPredef,  "0", NULL, 0, NULL },
        { "bronze",         CanvasMaterialPredef,  "1", NULL, 0, NULL },
        { "bronzePolished", CanvasMaterialPredef,  "2", NULL, 0, NULL },
        { "chrome",         CanvasMaterialPredef,  "3", NULL, 0, NULL },
        { "copper",         CanvasMaterialPredef,  "4", NULL, 0, NULL },
        { "copperPolished", CanvasMaterialPredef,  "5", NULL, 0, NULL },
        { "gold",           CanvasMaterialPredef,  "5", NULL, 0, NULL },
        { "goldPolished",   CanvasMaterialPredef,  "7", NULL, 0, NULL },
        { "pewter",         CanvasMaterialPredef,  "8", NULL, 0, NULL },
        { "silver",         CanvasMaterialPredef,  "9", NULL, 0, NULL },
        { "silverPolished", CanvasMaterialPredef, "10", NULL, 0, NULL },
    };

    static menu_struct PlasticData[] = {
        { "blackPlastic",   CanvasMaterialPredef, "11", NULL, 0, NULL },
        { "bluePlastic",    CanvasMaterialPredef, "12", NULL, 0, NULL },
        { "cyanPlastic",    CanvasMaterialPredef, "13", NULL, 0, NULL },
        { "greenPlastic",   CanvasMaterialPredef, "14", NULL, 0, NULL },
        { "greyPlastic",    CanvasMaterialPredef, "15", NULL, 0, NULL },
        { "pinkPlastic",    CanvasMaterialPredef, "16", NULL, 0, NULL },
        { "redPlastic",     CanvasMaterialPredef, "17", NULL, 0, NULL },
        { "whitePlastic",   CanvasMaterialPredef, "18", NULL, 0, NULL },
        { "yellowPlastic",  CanvasMaterialPredef, "19", NULL, 0, NULL },
    };

    static menu_struct RubberData[] = {
        { "blackRubber",    CanvasMaterialPredef, "20", NULL, 0, NULL },
        { "blueRubber",     CanvasMaterialPredef, "23", NULL, 0, NULL },
        { "cyanRubber",     CanvasMaterialPredef, "26", NULL, 0, NULL },
        { "greenRubber",    CanvasMaterialPredef, "21", NULL, 0, NULL },
        { "magentaRubber",  CanvasMaterialPredef, "24", NULL, 0, NULL },
        { "orangeRubber",   CanvasMaterialPredef, "27", NULL, 0, NULL },
        { "redRubber",      CanvasMaterialPredef, "22", NULL, 0, NULL },
        { "whiteRubber",    CanvasMaterialPredef, "25", NULL, 0, NULL },
        { "yellowRubber",   CanvasMaterialPredef, "28", NULL, 0, NULL },
    };

    static menu_struct StoneData[] = {
        { "emerald",        CanvasMaterialPredef, "29", NULL, 0, NULL },
        { "jade",           CanvasMaterialPredef, "32", NULL, 0, NULL },
        { "obsidian",       CanvasMaterialPredef, "30", NULL, 0, NULL },
        { "pearl",          CanvasMaterialPredef, "33", NULL, 0, NULL },
        { "ruby",           CanvasMaterialPredef, "31", NULL, 0, NULL },
        { "turquoise",      CanvasMaterialPredef, "34", NULL, 0, NULL },
    };

    static menu_struct OtherData[] = {
        { "default",        CanvasMaterialPredef, "35", NULL, 0, NULL },
        { "brown",          CanvasMaterialPredef, "36", NULL, 0, NULL },
        { "darkbrown",      CanvasMaterialPredef, "37", NULL, 0, NULL },
        { "gray",           CanvasMaterialPredef, "38", NULL, 0, NULL },
        { "glass",          CanvasMaterialPredef, "39", NULL, 0, NULL },
        { "green",          CanvasMaterialPredef, "40", NULL, 0, NULL },
        { "red",            CanvasMaterialPredef, "41", NULL, 0, NULL },
        { "sea",            CanvasMaterialPredef, "42", NULL, 0, NULL },
        { "white",          CanvasMaterialPredef, "43", NULL, 0, NULL },
    };

    /*
     * Describe the menu bar, giving only the names to appear in
     * the menu bar and pointers to each pulldown pane.
     */
    static menu_struct PulldownData[] = {
        { "Metal",   NULL, NULL, MetalData,   XtNumber(MetalData),   NULL },
        { "Plastic", NULL, NULL, PlasticData, XtNumber(PlasticData), NULL },
        { "Rubber",  NULL, NULL, RubberData,  XtNumber(RubberData),  NULL },
        { "Stone",   NULL, NULL, StoneData,   XtNumber(StoneData),   NULL },
        { "Other",   NULL, NULL, OtherData,   XtNumber(OtherData),   NULL },
    };

    Position x, y;

    Widget w_menu, w_f, w_quit;

    map_dialog( w_material_predef, &x, &y );
    w_control_material_predef = XtVaCreatePopupShell( "Predef Material Panel",
        transientShellWidgetClass,
        w_material_predef,
#ifndef USE_ATHENA
        XmNx, x,
        XmNy, y,
#else
        XtNx, x,
        XtNy, y,
#endif
        NULL );

    w_f = XtVaCreateManagedWidget( "w_f",
#ifndef USE_ATHENA
        xmFormWidgetClass, w_control_material_predef,
#ifdef XPM
        XmNbackgroundPixmap, groundPixmap,
#endif
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 5,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 5,
        XmNwidth, 360,
        XmNheight, 30,
#else
        formWidgetClass, w_control_material_predef,
#ifdef XPM
        XtNbackgroundPixmap, groundPixmap,
#endif
        XtNhorizDistance, 5,
        XtNvertDistance, 5,
#endif
        NULL );

    w_menu = XtVaCreateManagedWidget( "w_menu",
#ifndef USE_ATHENA
        xmRowColumnWidgetClass, w_f,
        XmNrowColumnType, XmMENU_BAR,
        XmNorientation, XmHORIZONTAL,
        XmNshadowThickness, 0,
        XmNhighlightThickness, 0,
        XmNwidth, 360,
        XmNheight, 30,
#ifdef XPM
        XmNbackgroundPixmap, groundPixmap,
#endif
#else
        boxWidgetClass, w_f,
        XtNorientation, XtorientHorizontal,
        XtNborderWidth, 0,
        XtNhighlightThickness, 0,
#ifdef XPM
        XtNbackgroundPixmap, groundPixmap,
#endif
#endif
        NULL );

    create_menus( NULL, w_menu, PulldownData, XtNumber( PulldownData ) );

    w_quit = XtVaCreateManagedWidget( "w_quit",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_f,
        XmNlabelString, XmStringCreateSimple( "Quit" ),
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_menu,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 5,
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 5,
        XmNwidth, strlen("Quit") * 10,
        XmNheight, 25,
#else
        commandWidgetClass, w_f,
        XtNlabel, "Quit",
        XtNfromHoriz, w_menu,
        XtNhorizDistance, 5,
        XtNvertDistance, 8,
#endif
        NULL );
    XtAddCallback( w_quit,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasMaterialPredefQuit,
#else
        XtNcallback, (XtCallbackProc)CanvasMaterialPredefQuit,
#endif
        (XtPointer)NULL );

    XtRealizeWidget( w_control_material_predef );
    XtPopup( w_control_material_predef, XtGrabNone );

#ifndef USE_ATHENA
    wm_delete_window = XmInternAtom( display, "WM_DELETE_WINDOW", False );
    XmAddWMProtocolCallback( w_control_material_predef, wm_delete_window,
        (XtCallbackProc)quit, NULL );
#endif
}


Boolean createVideoPanel()
{
    Position x, y;

    extern XtActionProc CanvasSavePPM(), CanvasSaveMPG(),
        CanvasStepBack(), CanvasPlay(), CanvasStepForw(), CanvasStop(),
        CanvasRecord(), CanvasVideoQuit();

    Widget w_vcr, w_sback, w_play, w_sforw, w_stop,
        w_record, w_savePPM, w_saveMPG,
        w_quit;

    Pixmap grayPixmap, grayBitmap;
    Pixmap sbackPixmap, playPixmap, sforwPixmap, stopPixmap, recPixmap;
    Pixmap sbackBitmap, playBitmap, sforwBitmap, stopBitmap, recBitmap;
    int status = 1;     /* !XpmSuccess */

#ifdef XPM
    XpmAttributes attributes;
    Pixmap pix;
#endif

    map_dialog( w_video, &x, &y );
    w_control_video = XtVaCreatePopupShell( "Video Panel",
        transientShellWidgetClass,
        w_video,
#ifndef USE_ATHENA
        XmNx, x,
        XmNy, y,
#else
        XtNx, x,
        XtNy, y,
#endif
        NULL );

    w_vcr = XtVaCreateManagedWidget( "w_vcr",
#ifndef USE_ATHENA
        xmFormWidgetClass, w_control_video,
#ifdef XPM
        XmNbackgroundPixmap, groundPixmap,
#endif
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 5,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 5,
        XmNwidth, 130,
        XmNheight, 140,
#else
        formWidgetClass, w_control_video,
#ifdef XPM
        XtNbackgroundPixmap, groundPixmap,
#endif
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
#endif
        NULL );

#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/gray.xpm", &grayPixmap, &pix, &attributes );
    errorMessageXPM( status, "Gray" );
#endif

    if( status != 0 /* XpmSuccess */ ) {
        #include "pixmaps/gray.xbm"
        grayBitmap = XCreatePixmapFromBitmapData( display,
            DefaultRootWindow( display ),
            gray_bits, gray_width, gray_height,
            BlackPixel( display, DefaultScreen(display) ),
            WhitePixel( display, DefaultScreen(display) ), 8 );
        grayPixmap = grayBitmap;
    }

#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/sback.xpm", &sbackPixmap, &pix, &attributes );
    errorMessageXPM( status, "Back" );
#endif

    if( status != 0 /* XpmSuccess */ ) {
        #include "pixmaps/sback.xbm"
        sbackBitmap = XCreatePixmapFromBitmapData( display,
            DefaultRootWindow( display ),
            sback_bits, sback_width, sback_height,
            BlackPixel( display, DefaultScreen(display) ),
            WhitePixel( display, DefaultScreen(display) ), 8 );
        sbackPixmap = sbackBitmap;
    }

    w_sback = XtVaCreateManagedWidget( "w_sback",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_vcr,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNwidth, 25,
        XmNheight, 25,
        XmNlabelType, XmPIXMAP,
        XmNlabelPixmap, sbackPixmap,
        XmNbackgroundPixmap, grayPixmap,
#else
        commandWidgetClass, w_vcr,
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
        XtNwidth, 32,
        XtNheight, 25,
        XtNbackgroundPixmap, sbackPixmap,
        XtNlabel, " ",
#endif
        NULL );
    XtAddCallback( w_sback,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasStepBack,
#else
        XtNcallback, (XtCallbackProc)CanvasStepBack,
#endif
        (XtPointer)NULL );

#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/play.xpm", &playPixmap, &pix, &attributes );
    errorMessageXPM( status, "Play" );
#endif

    if( status != 0 /* XpmSuccess */ ) {
        #include "pixmaps/play.xbm"
        playBitmap = XCreatePixmapFromBitmapData( display,
            DefaultRootWindow( display ),
            play_bits, play_width, play_height,
            BlackPixel( display, DefaultScreen(display) ),
            WhitePixel( display, DefaultScreen(display) ), 8 );
        playPixmap = playBitmap;
    }

    w_play = XtVaCreateManagedWidget( "w_play",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_vcr,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_sback,
        XmNleftOffset, 6,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNwidth, 25,
        XmNheight, 25,
        XmNlabelType, XmPIXMAP,
        XmNlabelPixmap, playPixmap,
        XmNbackgroundPixmap, grayPixmap,
#else
        commandWidgetClass, w_vcr,
        XtNfromHoriz, w_sback,
        XtNhorizDistance, 6,
        XtNvertDistance, 5,
        XtNwidth, 32,
        XtNheight, 25,
        XtNbackgroundPixmap, playPixmap,
        XtNlabel, " ",
#endif
        NULL );
    XtAddCallback( w_play,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasPlay,
#else
        XtNcallback, (XtCallbackProc)CanvasPlay,
#endif
        (XtPointer)NULL );

#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/sforw.xpm", &sforwPixmap, &pix, &attributes );
    errorMessageXPM( status, "Forw" );
#endif

    if( status != 0 /* XpmSuccess */ ) {
        #include "pixmaps/sforw.xbm"
        sforwBitmap = XCreatePixmapFromBitmapData( display,
            DefaultRootWindow( display ),
            sforw_bits, sforw_width, sforw_height,
            BlackPixel( display, DefaultScreen(display) ),
            WhitePixel( display, DefaultScreen(display) ), 8 );
        sforwPixmap = sforwBitmap;
    }

    w_sforw = XtVaCreateManagedWidget( "w_sforw",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_vcr,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_play,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNwidth, 25,
        XmNheight, 25,
        XmNlabelType, XmPIXMAP,
        XmNlabelPixmap, sforwPixmap,
        XmNbackgroundPixmap, grayPixmap,
#else
        commandWidgetClass, w_vcr,
        XtNfromHoriz, w_play,
        XtNhorizDistance, 5,
        XtNvertDistance, 5,
        XtNwidth, 32,
        XtNheight, 25,
        XtNbackgroundPixmap, sforwPixmap,
        XtNlabel, " ",
#endif
        NULL );
    XtAddCallback( w_sforw,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasStepForw,
#else
        XtNcallback, (XtCallbackProc)CanvasStepForw,
#endif
        (XtPointer)NULL );

#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/stop.xpm", &stopPixmap, &pix, &attributes );
    errorMessageXPM( status, "Stop" );
#endif

    if( status != 0 /* XpmSuccess */ ) {
        #include "pixmaps/stop.xbm"
        stopBitmap = XCreatePixmapFromBitmapData( display,
            DefaultRootWindow( display ),
            stop_bits, stop_width, stop_height,
            BlackPixel( display, DefaultScreen(display) ),
            WhitePixel( display, DefaultScreen(display) ), 8 );
        stopPixmap = stopBitmap;
    }

    w_stop = XtVaCreateManagedWidget( "w_stop",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_vcr,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_sforw,
        XmNleftOffset, 6,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNwidth, 25,
        XmNheight, 25,
        XmNlabelType, XmPIXMAP,
        XmNlabelPixmap, stopPixmap,
        XmNbackgroundPixmap, grayPixmap,
#else
        commandWidgetClass, w_vcr,
        XtNfromHoriz, w_sforw,
        XtNhorizDistance, 6,
        XtNvertDistance, 5,
        XtNwidth, 32,
        XtNheight, 25,
        XtNbackgroundPixmap, stopPixmap,
        XtNlabel, " ",
#endif
        NULL );
    XtAddCallback( w_stop,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasStop,
#else
        XtNcallback, (XtCallbackProc)CanvasStop,
#endif
        (XtPointer)NULL );

#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/rec.xpm", &recPixmap, &pix, &attributes );
    errorMessageXPM( status, "Record" );
#endif

    if( status != 0 /* XpmSuccess */ ) {
        #include "pixmaps/rec.xbm"
        recBitmap = XCreatePixmapFromBitmapData( display,
            DefaultRootWindow( display ),
            rec_bits, rec_width, rec_height,
            BlackPixel( display, DefaultScreen(display) ),
            WhitePixel( display, DefaultScreen(display) ), 8 );
        recPixmap = recBitmap;
    }

    w_record = XtVaCreateManagedWidget( "w_record",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_vcr,
        XmNlabelString, XmStringCreateSimple( " REC" ),
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_play,
        XmNtopOffset, 5,
        XmNwidth, strlen(" REC ") * 9,
        XmNheight, 25,
/*      XmNlabelType, XmPIXMAP, */
/*      XmNlabelPixmap, recPixmap, */
/*      XmNbackgroundPixmap, grayPixmap, */
        XmNbackgroundPixmap, recPixmap,
#else
        commandWidgetClass, w_vcr,
        XtNlabel, " REC",
        XtNhorizDistance, 5,
        XtNfromVert, w_play,
        XtNvertDistance, 5,
        XtNwidth, 44,
        XtNbackgroundPixmap, recPixmap,
#endif
        NULL );
    XtAddCallback( w_record,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasRecord,
#else
        XtNcallback, (XtCallbackProc)CanvasRecord,
#endif
        (XtPointer)NULL );

    w_savePPM = XtVaCreateManagedWidget( "w_savePPM",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_vcr,
        XmNlabelString, XmStringCreateSimple( "PPM" ),
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_record,
        XmNtopOffset, 5,
        XmNwidth, strlen("PPM ") * 10,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_vcr,
        XtNlabel, "PPM",
        XtNhorizDistance, 5,
        XtNfromVert, w_record,
        XtNvertDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_savePPM,
#ifndef USE_ATHENA
        XmNactivateCallback,(XtCallbackProc)CanvasSavePPM,
#else
        XtNcallback,(XtCallbackProc)CanvasSavePPM,
#endif
        (XtPointer)NULL );

    w_saveMPG = XtVaCreateManagedWidget( "w_saveMPG",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_vcr,
        XmNlabelString, XmStringCreateSimple( "MPG" ),
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_savePPM,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_record,
        XmNtopOffset, 5,
        XmNwidth, strlen("MPG ") * 10,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_vcr,
        XtNlabel, "MPG",
        XtNfromHoriz, w_savePPM,
        XtNhorizDistance, 5,
        XtNfromVert, w_record,
        XtNvertDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_saveMPG,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasSaveMPG,
#else
        XtNcallback, (XtCallbackProc)CanvasSaveMPG,
#endif
        (XtPointer)NULL );

    w_quit = XtVaCreateManagedWidget( "w_quit",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_vcr,
        XmNlabelString, XmStringCreateSimple( "Quit" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_saveMPG,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 5,
        XmNwidth, strlen("Quit") * 10,
        XmNheight, strlen("Quit") * 10,
#else
        commandWidgetClass, w_vcr,
        XtNlabel, "Quit",
        XtNfromVert, w_saveMPG,
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
#endif
        NULL );
    XtAddCallback( w_quit,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasVideoQuit,
#else
        XtNcallback, (XtCallbackProc)CanvasVideoQuit,
#endif
        (XtPointer)NULL );

    XtRealizeWidget( w_control_video );

#ifndef USE_ATHENA
    wm_delete_window = XmInternAtom( display, "WM_DELETE_WINDOW", False );
    XmAddWMProtocolCallback( w_control_video, wm_delete_window,
        (XtCallbackProc)quit, NULL );
#endif
    return True;
}


Boolean createPrintPanel()
{
    Position x, y;
    static XtCallbackRec callbacks[1];

    extern XtActionProc CanvasHardcopy(), CanvasPrintQuit();

    Widget w_hard, w_xwd, w_filter, w_print, w_ok, w_quit;

    Pixmap grayPixmap, grayBitmap;
    int status = 1;     /* !XpmSuccess */

#ifdef XPM
    XpmAttributes attributes;
    Pixmap recPixmap, pix;
    Pixmap sbackPixmap, playPixmap, sforwPixmap, stopPixmap;
#endif

    map_dialog( w_hardcopy, &x, &y );
    w_control_print = XtVaCreatePopupShell( "Print Panel",
        transientShellWidgetClass,
        w_hardcopy,
#ifndef USE_ATHENA
        XmNx, x,
        XmNy, y,
#else
        XtNx, x,
        XtNy, y,
#endif
        NULL );

    w_hard = XtVaCreateManagedWidget( "w_hard",
#ifndef USE_ATHENA
        xmFormWidgetClass, w_control_print,
#ifdef XPM
        XmNbackgroundPixmap, groundPixmap,
#endif
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 5,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 5,
        XmNwidth, 255,
        XmNheight, 165,
#else
        formWidgetClass, w_control_print,
#ifdef XPM
        XtNbackgroundPixmap, groundPixmap,
#endif
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
#endif
        NULL );

#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/gray.xpm", &grayPixmap, &pix, &attributes );
    errorMessageXPM( status, "Gray" );
#endif

    if( status != 0 /* XpmSuccess */ ) {
        #include "pixmaps/gray.xbm"
        grayBitmap = XCreatePixmapFromBitmapData( display,
            DefaultRootWindow( display ),
            gray_bits, gray_width, gray_height,
            BlackPixel( display, DefaultScreen(display) ),
            WhitePixel( display, DefaultScreen(display) ), 8 );
        grayPixmap = grayBitmap;
    }

    w_xwd = XtVaCreateManagedWidget( "w_xwd",
#ifndef USE_ATHENA
        xmLabelWidgetClass, w_hard,
        XmNlabelString, XmStringCreateSimple( "XWD:" ),
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
        XmNheight, 31,
#else
        labelWidgetClass, w_hard,
        XtNlabel, "XWD:",
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );

    w_xwd_text = XtVaCreateManagedWidget( "w_xwd_text",
#ifndef USE_ATHENA
        xmTextFieldWidgetClass, w_hard,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_xwd,
        XmNleftOffset, 8,
        XmNvalue, XwdCommand,
#else
        asciiTextWidgetClass, w_hard,
        XtNeditType, XawtextEdit,
        XtNtype, XawAsciiString,
        XtNuseStringInPlace, True,
        XtNresizable, True,
        XtNresize, XawtextResizeWidth,
        XtNright, XtChainRight,
        XtNstring, XwdCommand,
        XtNlength, 30,
        XtNwidth, 11*strlen(XwdCommand),
        XtNinsertPosition, strlen(XwdCommand),
        XtNvertDistance, 5,
        XtNfromHoriz, w_xwd,
        XtNhorizDistance, 35,
        XtNheight, 21,
#endif
        NULL );

    w_filter = XtVaCreateManagedWidget( "w_filter",
#ifndef USE_ATHENA
        xmLabelWidgetClass, w_hard,
        XmNlabelString, XmStringCreateSimple( "Filter:" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_xwd,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
        XmNheight, 31,
#else
        labelWidgetClass, w_hard,
        XtNlabel, "Filter:",
        XtNfromVert, w_xwd,
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );

    w_filter_text = XtVaCreateManagedWidget( "w_filter_text",
#ifndef USE_ATHENA
        xmTextFieldWidgetClass, w_hard,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_xwd_text,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_filter,
        XmNleftOffset, 5,
        XmNvalue, FilterCommand,
#else
        asciiTextWidgetClass, w_hard,
        XtNeditType, XawtextEdit,
        XtNtype, XawAsciiString,
        XtNuseStringInPlace, True,
        XtNresizable, True,
        XtNresize, XawtextResizeWidth,
        XtNright, XtChainRight,
        XtNstring, FilterCommand,
        XtNlength, 30,
        XtNwidth, 11*strlen(FilterCommand),
        XtNinsertPosition, strlen(FilterCommand),
        XtNfromVert, w_xwd_text,
        XtNvertDistance, 5,
        XtNfromHoriz, w_filter,
        XtNhorizDistance, 5,
        XtNheight, 21,
#endif
        NULL );

    w_print = XtVaCreateManagedWidget( "w_print",
#ifndef USE_ATHENA
        xmLabelWidgetClass, w_hard,
        XmNlabelString, XmStringCreateSimple( "Print:" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_filter,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
        XmNheight, 31,
#else
        labelWidgetClass, w_hard,
        XtNlabel, "Print:",
        XtNfromVert, w_filter,
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );

    w_print_text = XtVaCreateManagedWidget( "w_print_text",
#ifndef USE_ATHENA
        xmTextFieldWidgetClass, w_hard,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_filter_text,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_print,
        XmNleftOffset, 7,
        XmNvalue, PrintCommand,
#else
        asciiTextWidgetClass, w_hard,
        XtNeditType, XawtextEdit,
        XtNtype, XawAsciiString,
        XtNuseStringInPlace, True,
        XtNresizable, True,
        XtNresize, XawtextResizeWidth,
        XtNright, XtChainRight,
        XtNstring, PrintCommand,
        XtNlength, 30,
        XtNwidth, 11*strlen(PrintCommand),
        XtNinsertPosition, strlen(PrintCommand),
        XtNfromVert, w_filter_text,
        XtNvertDistance, 5,
        XtNfromHoriz, w_print,
        XtNhorizDistance, 15,
        XtNheight, 21,
#endif
        NULL );

    w_ok = XtVaCreateManagedWidget( "w_ok",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_hard,
        XmNlabelString, XmStringCreateSimple( "OK" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_print,
        XmNtopOffset, 10,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
        XmNwidth, 25,
        XmNheight, 25,
#else
        commandWidgetClass, w_hard,
        XtNlabel, "OK",
        XtNfromVert, w_print,
        XtNvertDistance, 10,
        XtNhorizDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_ok,
#ifndef USE_ATHENA 
        XmNactivateCallback, (XtCallbackProc)CanvasHardcopy,
#else
        XtNcallback, (XtCallbackProc)CanvasHardcopy,
#endif
        (XtPointer)NULL );

    w_quit = XtVaCreateManagedWidget( "w_quit",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_hard,
        XmNlabelString, XmStringCreateSimple( "Quit" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_print_text,
        XmNtopOffset, 10,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_ok,
        XmNleftOffset, 10,
        XmNwidth, strlen("Quit") * 10,
        XmNheight, strlen("Quit") * 10,
#else
        commandWidgetClass, w_hard,
        XtNlabel, "Quit",
        XtNfromVert, w_print,
        XtNvertDistance, 10,
        XtNfromHoriz, w_ok,
        XtNhorizDistance, 10,
#endif
        NULL );
    XtAddCallback( w_quit,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasPrintQuit,
#else
        XtNcallback, (XtCallbackProc)CanvasPrintQuit,
#endif
        (XtPointer)NULL );

    XtRealizeWidget( w_control_print );

#ifndef USE_ATHENA
    wm_delete_window = XmInternAtom( display, "WM_DELETE_WINDOW", False );
    XmAddWMProtocolCallback( w_control_print, wm_delete_window,
        (XtCallbackProc)quit, NULL );
#endif
    return True;
}


void createControlPanel()
{
    Widget w_box, w_form, w_form1,
           w_head, w_headUp, w_headDo, w_headLe, w_headRi,
           w_quit, w_pex;
    Position x, y;

    Arg args[10];
    register int n=0;
    Widget quadric, function, blob, lSystem, revolution, extrusion,
           simple, platonic, polyhedra, fractal, shell;

    extern int no_move;

    Pixmap grayPixmap, grayBitmap;
    Pixmap pexPixmap,  pexBitmap;
    int status = 1;     /* !XpmSuccess */

#ifdef XPM
    XpmAttributes attributes;
    Pixmap wirePixmap, colorPixmap, flybyPixmap;
    Pixmap pix;
    Pixmap upPixmap, doPixmap, lePixmap, riPixmap;
#endif

    extern int object_defined;
    extern XtActionProc CanvasAbout(), CanvasReset(), CanvasQuit(),
        CanvasWireframe(), CanvasCull(), CanvasEdge(), CanvasShadow(),
        CanvasColor(), CanvasMaterial(), CanvasLight(),
        CanvasFlybyCr(), CanvasFlybyEx(), CanvasAnimateCr(), CanvasAnimateEx(),
        CanvasMorph(), CanvasSpin(), CanvasMelt(),
        CanvasHardcopy(), CanvasPrint(), CanvasVideo(),
        CanvasUp(), CanvasDown(), CanvasLeft(), CanvasRight();

    static XtCallbackRec callbacks[] =
    {
        {(XtCallbackProc) NULL, (XtPointer) NULL},
        {(XtCallbackProc) NULL, (XtPointer) NULL},
    };

    extern void create_menus();
    extern XtActionProc changeENV(), createOFFObject();
    Widget w_menu;

    static menu_struct ObjectData[] = {
        { "Simple",     changeENV, "Smpl", NULL, 0, NULL, createOFFObject },
        { "Quadric",    changeENV, "Quad", NULL, 0, NULL, createOFFObject },
        { "Function",   changeENV, "Func", NULL, 0, NULL, createOFFObject },
        { "Blob",       changeENV, "Blob", NULL, 0, NULL, createOFFObject },
        { "Revolution", changeENV, "Revl", NULL, 0, NULL, createOFFObject },
        { "Extrusion",  changeENV, "Extr", NULL, 0, NULL, createOFFObject },
        { "Platonic",   changeENV, "Plnc", NULL, 0, NULL, createOFFObject },
        { "Polyhedra",  changeENV, "Poly", NULL, 0, NULL, createOFFObject },
        { "Fractal",    changeENV, "Frac", NULL, 0, NULL, createOFFObject },
        { "L-System",   changeENV, "Tree", NULL, 0, NULL, createOFFObject },
        { "Sea Shell",  changeENV, "Shel", NULL, 0, NULL, createOFFObject },
    };

    static menu_struct SceneData[] = {
        { "Load", NULL, NULL, NULL, 0, NULL },
    };

    static menu_struct FileData[] = {
#ifndef USE_ATHENA
        { "Create", NULL,          NULL, ObjectData, XtNumber(ObjectData), NULL },
#endif
        { "Load",   loadOFFObject, NULL, NULL,       0,                    NULL },
        { "Save",   saveOFFObject, NULL, NULL,       0,                    NULL },
        { "Quit",   quitOFFObject, NULL, NULL,       0,                    NULL },
    };

    static menu_struct PulldownData[] = {
        { "File",   NULL, NULL, FileData,   XtNumber(FileData),   NULL },
#ifdef USE_ATHENA
        { "Create", NULL, NULL, ObjectData, XtNumber(ObjectData), NULL },
#endif
#if( 0 ) 
        { "Scene",  NULL, NULL, SceneData,  XtNumber(SceneData),  NULL },
#endif
    };

#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/background.xpm", &groundPixmap, &pix, &attributes );
    errorMessageXPM( status, "Background" );
#endif

    if( status != 0 /* XpmSuccess */ ) {
        #include "pixmaps/background.xbm"
        groundBitmap = XCreatePixmapFromBitmapData( display,
            DefaultRootWindow( display ),
            background_bits, background_width, background_height,
            BlackPixel( display, DefaultScreen(display) ),
            WhitePixel( display, DefaultScreen(display) ), 8 );
        groundPixmap = groundBitmap;
    }

    w_control = XtVaCreatePopupShell( "PEX-View Object",
        topLevelShellWidgetClass,
        toplevel,
#ifndef USE_ATHENA
        XmNiconPixmap, geom_icon,
        XmNautoUnmanage, False,
        XmNdefaultPosition, False,
        XmNdeleteResponse, XmDO_NOTHING,
        XmNkeyboardFocusPolicy, XmPOINTER,
        XmNwidth,    250,
        XmNminWidth, 250,
        XmNmaxWidth, 250,
        XmNheight,    13 * 31 + 35,
        XmNminHeight, 13 * 31 + 35,
        XmNmaxHeight, 13 * 31 + 35,
        XmNgeometry, "=+50+150",
#else
        XtNiconPixmap, geom_icon,
        XtNgeometry, "=+50+150",
#endif
        NULL );

#if( 0 )
    XtVaSetValues( w_control,
        XmNmwmDecorations, MWM_DECOR_BORDER | MWM_DECOR_TITLE |
            MWM_DECOR_MINIMIZE,
        XmNmwmFunctions, MWM_FUNC_MOVE | MWM_FUNC_MINIMIZE | MWM_FUNC_CLOSE,
            NULL );
#endif

#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/gray.xpm", &grayPixmap, &pix, &attributes );
    errorMessageXPM( status, "Gray" );
#endif

    if( status != 0 /* XpmSuccess */ ) {
        #include "pixmaps/gray.xbm"
        grayBitmap = XCreatePixmapFromBitmapData( display,
            DefaultRootWindow( display ),
            gray_bits, gray_width, gray_height,
            BlackPixel( display, DefaultScreen(display) ),
            WhitePixel( display, DefaultScreen(display) ), 8 );
        grayPixmap = grayBitmap;
    }

    w_box = XtVaCreateManagedWidget( "w_box",
#ifndef USE_ATHENA
        xmFormWidgetClass, w_control,
        XmNwidth,    250,
        XmNminWidth, 250,
        XmNmaxWidth, 250,
        XmNheight,    13 * 31 + 35,
        XmNminHeight, 13 * 31 + 35,
        XmNmaxHeight, 13 * 31 + 35,
#ifdef XPM
        XmNbackgroundPixmap, groundPixmap,
#endif
#else
        formWidgetClass, w_control,
#ifdef XPM
        XtNbackgroundPixmap, groundPixmap,
#endif
#endif
        NULL );

    w_form = XtVaCreateManagedWidget( "w_form",
#ifndef USE_ATHENA
        xmFormWidgetClass, w_box,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 1,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 1,
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 1,
        XmNbottomOffset, 1,
        XmNwidth,    250,
        XmNminWidth, 250,
        XmNmaxWidth, 250,
        XmNheight,    34,
        XmNminHeight, 34,
        XmNmaxHeight, 34,
#else
        formWidgetClass, w_box,
        XtNborderWidth, 0,
        XtNwidth,    210,
        XtNheight,    34,
#endif
        NULL );

    w_menu = XtVaCreateManagedWidget( "w_menu",
#ifndef USE_ATHENA
        xmRowColumnWidgetClass, w_form,
        XmNrowColumnType, XmMENU_BAR,
        XmNorientation, XmHORIZONTAL,
        XmNshadowThickness, 0,
        XmNhighlightThickness, 0,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 5,
        XmNwidth, 300,
        XmNheight, 30,
#else
        boxWidgetClass, w_form,
        XtNorientation, XtorientHorizontal,
        XtNborderWidth, 0,
        XtNhighlightThickness, 0,
#endif
        NULL );

    create_menus( NULL, w_menu, PulldownData, XtNumber( PulldownData ) );

    w_about = XtVaCreateManagedWidget( "w_about",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form,
        XmNlabelString, XmStringCreateSimple( "About" ),
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 5,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 5,
        XmNwidth, strlen("About") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form,
        XtNlabel, "About",
        XtNvertDistance, 8,
        XtNhorizDistance, 210,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_about,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasAbout,
#else
        XtNcallback, (XtCallbackProc)CanvasAbout,
#endif
        (XtPointer)NULL );

    w_form1 = XtVaCreateManagedWidget( "w_form1",
#ifndef USE_ATHENA
        xmFormWidgetClass, w_box,
#ifdef XPM
        XmNbackgroundPixmap, groundPixmap,
#endif
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_form,
        XmNleftAttachment, XmATTACH_FORM,
        XmNrightAttachment, XmATTACH_FORM,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNwidth,    250,
        XmNminWidth, 250,
        XmNmaxWidth, 250,
        XmNheight,    13 * 31,
        XmNminHeight, 13 * 31,
        XmNmaxHeight, 13 * 31,
#else
        formWidgetClass, w_box,
#ifdef XPM
        XtNbackgroundPixmap, groundPixmap,
#endif
        XtNfromVert, w_form,
        XtNborderWidth, 0,
#endif
        NULL );

    w_reset = XtVaCreateManagedWidget( "w_reset",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Reset" ),
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNwidth, strlen("Reset") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Reset",
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_reset,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasReset,
#else
        XtNcallback, (XtCallbackProc)CanvasReset,
#endif
        (XtPointer)NULL );

    callbacks[0].callback = (XtCallbackProc) createVisualPanel;
    callbacks[0].closure  = (XtPointer) NULL;
    w_visual = XtVaCreateManagedWidget( "w_visual",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Visual" ),
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 5,
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 5,
        XmNwidth, strlen("Visual") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
        XmNactivateCallback, callbacks,
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Visual",
        XtNfromHoriz, w_reset,
        XtNhorizDistance, 135,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
        XtNcallback, callbacks,
#endif
        NULL );

#if( 0 ) 
#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/wire.xpm", &wirePixmap, &pix, &attributes );
    errorMessageXPM( status, "Wire" );
#endif
#endif
    w_wireframe = XtVaCreateManagedWidget( "w_wireframe",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Solid" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_reset,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNrecomputeSize, False,
        XmNwidth, strlen("Solid ") * 11,
        XmNheight, 25,
#ifdef XPM
        /* XmNbackgroundPixmap, wirePixmap, */
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Solid",
        XtNfromVert, w_reset,
        XtNvertDistance, 5,
        XtNresize, False,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_wireframe,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasWireframe,
#else
        XtNcallback, (XtCallbackProc)CanvasWireframe,
#endif
        (XtPointer)NULL );

    w_cull = XtVaCreateManagedWidget( "w_cull",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Cull On" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_reset,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_wireframe,
        XmNleftOffset, 5,
        XmNrecomputeSize, False,
        XmNwidth, strlen("CullOff") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Cull On",
        XtNfromVert, w_reset,
        XtNvertDistance, 5,
        XtNfromHoriz, w_wireframe,
        XtNhorizDistance, 5,
        XtNresize, False,
        XtNwidth, strlen("Cull Off") * 11,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_cull,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasCull,
#else
        XtNcallback, (XtCallbackProc)CanvasCull,
#endif
        (XtPointer)NULL );

    w_edge = XtVaCreateManagedWidget( "w_edge",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Edges Off" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_reset,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_cull,
        XmNleftOffset, 5,
        XmNrecomputeSize, False,
        XmNwidth, strlen("EdgesOff") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Edges Off",
        XtNfromVert, w_reset,
        XtNvertDistance, 5,
        XtNfromHoriz, w_cull,
        XtNhorizDistance, 5,
        XtNresize, False,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_edge,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasEdge,
#else
        XtNcallback, (XtCallbackProc)CanvasEdge,
#endif
        (XtPointer)NULL );

    w_shadow = XtVaCreateManagedWidget( "w_shadow",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Shadow" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_wireframe,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNwidth, strlen("Shadow") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Shadow",
        XtNfromVert, w_wireframe,
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_shadow,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasShadow,
#else
        XtNcallback, (XtCallbackProc)CanvasShadow,
#endif
        (XtPointer)NULL );

    w_morph = XtVaCreateManagedWidget( "w_morph",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Morph" ),
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_shadow,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_wireframe,
        XmNtopOffset, 5,
        XmNwidth, strlen("Morph ") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Morph",
        XtNfromHoriz, w_shadow,
        XtNhorizDistance, 5,
        XtNfromVert, w_wireframe,
        XtNvertDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_morph,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasMorph,
#else
        XtNcallback, (XtCallbackProc)CanvasMorph,
#endif
        (XtPointer)NULL );

#if( 0 ) 
    w_spin = XtVaCreateManagedWidget( "w_spin",
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Auto Spin" ),
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_morph,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_wireframe,
        XmNtopOffset, 5,
        XmNwidth, strlen("AutoSpin") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
        NULL );
    XtAddCallback( w_spin, XmNactivateCallback, (XtCallbackProc)CanvasSpin,
        (XtPointer)NULL );
#endif		/* spin */

    w_melt = XtVaCreateManagedWidget( "w_melt",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Melt" ),
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_morph,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_wireframe,
        XmNtopOffset, 5,
        XmNwidth, strlen("Melt ") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Melt",
        XtNfromHoriz, w_morph,
        XtNhorizDistance, 5,
        XtNfromVert, w_wireframe,
        XtNvertDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_melt,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasMelt,
#else
        XtNcallback, (XtCallbackProc)CanvasMelt,
#endif
        (XtPointer)NULL );

    w_vanish = XtVaCreateManagedWidget( "w_vanish",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Vanish" ),
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_shadow,
        XmNtopOffset, 5,
        XmNwidth, strlen("Vanish ") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Vanish",
        XtNhorizDistance, 5,
        XtNfromVert, w_shadow,
        XtNvertDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_vanish,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasVanish,
#else
        XtNcallback, (XtCallbackProc)CanvasVanish,
#endif
        (XtPointer)NULL );

#if( 0 ) 
#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/color.xpm", &colorPixmap, &pix, &attributes );
    errorMessageXPM( status, "Color" );
#endif
#endif
    w_color = XtVaCreateManagedWidget( "w_color",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Set Color" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_vanish,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNwidth, strlen("SetColor") * 11,
        XmNheight, 25,
#ifdef XPM
        /* XmNbackgroundPixmap, colorPixmap, */
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Set Color",
        XtNfromVert, w_vanish,
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_color,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasColor,
#else
        XtNcallback, (XtCallbackProc)CanvasColor,
#endif
        (XtPointer)NULL );

    w_light = XtVaCreateManagedWidget( "w_light",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Set Light" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_vanish,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_color,
        XmNleftOffset, 5,
        XmNwidth, strlen("SetLight") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Set Light",
        XtNfromVert, w_vanish,
        XtNvertDistance, 5,
        XtNfromHoriz, w_color,
        XtNhorizDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_light,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasLight,
#else
        XtNcallback, (XtCallbackProc)CanvasLight,
#endif
        (XtPointer)NULL );

    w_material = XtVaCreateManagedWidget( "w_material",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Set Material" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_color,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNwidth, strlen("SetMaterial") * 10,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Set Material",
        XtNfromVert, w_color,
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_material,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasMaterial,
#else
        XtNcallback, (XtCallbackProc)CanvasMaterial,
#endif
        (XtPointer)NULL );

    callbacks[0].callback = (XtCallbackProc) createMaterialPredefPanel;
    callbacks[0].closure  = (XtPointer) NULL;
    w_material_predef = XtVaCreateManagedWidget( "w_material_predef",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Use Material" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_color,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_material,
        XmNleftOffset, 5,
        XmNwidth, strlen("UseMaterial") * 10,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
        XmNactivateCallback, callbacks,
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Use Material",
        XtNfromVert, w_color,
        XtNvertDistance, 5,
        XtNfromHoriz, w_material,
        XtNhorizDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
        XtNcallback, callbacks,
#endif
        NULL );

    w_view = XtVaCreateManagedWidget( "w_view",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Set View" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_material,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNwidth, strlen("SetView") * 10,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Set View",
        XtNfromVert, w_material,
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_view,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasView,
#else
        XtNcallback, (XtCallbackProc)CanvasView,
#endif
        (XtPointer)NULL );

    w_depthCue = XtVaCreateManagedWidget( "w_depthCue",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Set Depth Cue" ),
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_material,
        XmNtopOffset, 5,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_view,
        XmNleftOffset, 5,
        XmNwidth, strlen("SetDepthCue") * 10,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Set Depth Cue",
        XtNfromVert, w_material,
        XtNfromHoriz, w_view,
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_depthCue,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasDepthCue,
#else
        XtNcallback, (XtCallbackProc)CanvasDepthCue,
#endif
        (XtPointer)NULL );

#if( 0 ) 
#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/flyby.xpm", &flybyPixmap, &pix, &attributes );
    errorMessageXPM( status, "Flyby" );
#endif
#endif

    w_flybycr = XtVaCreateManagedWidget( "w_flybycr",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Create Fly by" ),
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_view,
        XmNtopOffset, 5,
        XmNwidth, strlen("CreateFlyby") * 10,
        XmNheight, 25,
#ifdef XPM
        /* XmNbackgroundPixmap, flybyPixmap, */
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Create Fly by",
        XtNhorizDistance, 5,
        XtNfromVert, w_view,
        XtNvertDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_flybycr,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasFlybyCr,
#else
        XtNcallback, (XtCallbackProc)CanvasFlybyCr,
#endif
        (XtPointer)NULL );
    XtSetSensitive( w_flybycr, False );

    w_flybyex = XtVaCreateManagedWidget( "w_flybyex",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Fly by" ),
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_flybycr,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_view,
        XmNtopOffset, 5,
        XmNwidth, strlen("Fly by") * 11,
        XmNheight, 25,
#ifdef XPM
        /* XmNbackgroundPixmap, flybyPixmap, */
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Fly by",
        XtNhorizDistance, 5,
        XtNfromVert, w_view,
        XtNvertDistance, 5,
        XtNfromHoriz, w_flybycr,
        XtNhorizDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_flybyex,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasFlybyEx,
#else
        XtNcallback, (XtCallbackProc)CanvasFlybyEx,
#endif
        (XtPointer)NULL );

    callbacks[0].callback = (XtCallbackProc) CanvasAnimateCr;
    callbacks[0].closure  = (XtPointer) NULL;
    w_animcr = XtVaCreateManagedWidget( "w_animcr",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Create Anim" ),
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_flybycr,
        XmNtopOffset, 5,
        XmNwidth, strlen("CreateAnim") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
        XmNactivateCallback, callbacks,
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Create Anim",
        XtNhorizDistance, 5,
        XtNfromVert, w_flybycr,
        XtNvertDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
        XtNcallback, callbacks,
#endif
        NULL );

    callbacks[0].callback = (XtCallbackProc) CanvasAnimateEx;
    callbacks[0].closure  = (XtPointer) NULL;
    w_animex = XtVaCreateManagedWidget( "w_animex",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Animate" ),
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_animcr,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_flybyex,
        XmNtopOffset, 5,
        XmNwidth, strlen("Animate") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
        XmNactivateCallback, callbacks,
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Animate",
        XtNfromHoriz, w_animcr,
        XtNhorizDistance, 5,
        XtNfromVert, w_flybyex,
        XtNvertDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
        XtNcallback, callbacks,
#endif
        NULL );

#if( 0 )
    w_head = XtVaCreateManagedWidget( "w_head", xmPushButtonWidgetClass,
        w_form1,
        XmNlabelString, XmStringCreateSimple( "Head:" ),
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_flybyex,
        XmNtopOffset, 5,
        XmNwidth, strlen("Head:") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
        NULL );

#if( 0 ) 
#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/left.xpm", &lePixmap, &pix, &attributes );
    errorMessageXPM( status, "Left" );
#endif
#endif
    w_headLe = XtVaCreateManagedWidget( "w_headLe", xmPushButtonWidgetClass,
        w_form1,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_head,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_head,
        XmNtopOffset, 5,
#ifdef XPM
        XmNlabelType, XmPIXMAP,
        XmNlabelPixmap, lePixmap,
        XmNbackgroundPixmap, lePixmap,
#else
        XmNlabelString, XmStringCreateSimple( "Le" ),
#endif
        XmNwidth, 25,
        XmNheight, 25,
        NULL );
    XtAddCallback( w_headLe, XmNactivateCallback, (XtCallbackProc)CanvasLeft,
        (XtPointer)NULL );

#if( 0 ) 
#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/up.xpm", &upPixmap, &pix, &attributes );
    errorMessageXPM( status, "Up" );
#endif
#endif
    w_headUp = XtVaCreateManagedWidget( "w_headUp", xmPushButtonWidgetClass,
        w_form1,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_headLe,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_flybyex,
        XmNtopOffset, 5,
#ifdef XPM
        XmNlabelType, XmPIXMAP,
        XmNlabelPixmap, upPixmap,
        XmNbackgroundPixmap, upPixmap,
#else
        XmNlabelString, XmStringCreateSimple( "Up" ),
#endif
        XmNwidth, 25,
        XmNheight, 25,
        NULL );
    XtAddCallback( w_headUp, XmNactivateCallback, (XtCallbackProc)CanvasUp,
        (XtPointer)NULL );

#if( 0 ) 
#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/down.xpm", &doPixmap, &pix, &attributes );
    errorMessageXPM( status, "Down" );
#endif
#endif
    w_headDo = XtVaCreateManagedWidget( "w_headDo", xmPushButtonWidgetClass,
        w_form1,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_headLe,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_headLe,
        XmNtopOffset, 5,
#ifdef XPM
        XmNlabelType, XmPIXMAP,
        XmNlabelPixmap, doPixmap,
        XmNbackgroundPixmap, doPixmap,
#else
        XmNlabelString, XmStringCreateSimple( "Do" ),
#endif
        XmNwidth, 25,
        XmNheight, 25,
        NULL );
    XtAddCallback( w_headDo, XmNactivateCallback, (XtCallbackProc)CanvasDown,
        (XtPointer)NULL );

#if( 0 ) 
#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/right.xpm", &riPixmap, &pix, &attributes );
    errorMessageXPM( status, "Right" );
#endif
#endif
    w_headRi = XtVaCreateManagedWidget( "w_headRi", xmPushButtonWidgetClass,
        w_form1,
        XmNleftAttachment, XmATTACH_WIDGET,
        XmNleftWidget, w_headDo,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_headUp,
        XmNtopOffset, 5,
#ifdef XPM
        XmNlabelType, XmPIXMAP,
        XmNlabelPixmap, riPixmap,
        XmNbackgroundPixmap, riPixmap,
#else
        XmNlabelString, XmStringCreateSimple( "Ri" ),
#endif
        XmNwidth, 25,
        XmNheight, 25,
        NULL );
    XtAddCallback( w_headRi, XmNactivateCallback, (XtCallbackProc)CanvasRight,
        (XtPointer)NULL );
#endif

    w_video = XtVaCreateManagedWidget( "w_video",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Video" ),
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_animcr,
        XmNtopOffset, 5,
        XmNwidth, strlen("Video") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Video",
        XtNhorizDistance, 5,
        XtNfromVert, w_animcr,
        XtNvertDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_video,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasVideo,
#else
        XtNcallback, (XtCallbackProc)CanvasVideo,
#endif
        (XtPointer)NULL );

    w_hardcopy = XtVaCreateManagedWidget( "w_hardcopy",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( "Hardcopy" ),
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_video,
        XmNtopOffset, 5,
        XmNwidth, strlen("Hardcopy") * 11,
        XmNheight, 25,
#ifdef XPM
        XmNbackgroundPixmap, grayPixmap,
#endif
#else
        commandWidgetClass, w_form1,
        XtNlabel, "Hardcopy",
        XtNhorizDistance, 5,
        XtNfromVert, w_video,
        XtNvertDistance, 5,
#ifdef XPM
        XtNbackgroundPixmap, grayPixmap,
#endif
#endif
        NULL );
    XtAddCallback( w_hardcopy,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasPrint,
#else
        XtNcallback, (XtCallbackProc)CanvasPrint,
#endif
        (XtPointer)NULL );

#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/quit.xpm", &quitPixmap, &pix, &attributes );
    errorMessageXPM( status, "Quit" );
#endif

    if( status != 0 /* XpmSuccess */ ) {
        #include "pixmaps/quit.xbm"
        quitBitmap = XCreatePixmapFromBitmapData( display,
            DefaultRootWindow( display ),
            quit_bits, quit_width, quit_height,
            BlackPixel( display, DefaultScreen(display) ),
            WhitePixel( display, DefaultScreen(display) ), 8 );
        quitPixmap = quitBitmap;
    }

    w_quit = XtVaCreateManagedWidget( "w_quit",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
#ifdef XPM
        XmNlabelType, XmPIXMAP,
        XmNlabelPixmap, quitPixmap,
        XmNbackgroundPixmap, quitPixmap,
        XmNlabelString, XmStringCreateSimple( " " ),
#else
        XmNlabelString, XmStringCreateSimple( "Quit" ),
#endif
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 5,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, w_hardcopy,
        XmNtopOffset, 10,
        XmNwidth, 42,
        XmNheight, 42,
#else
        commandWidgetClass, w_form1,
#ifdef XPM
        XtNbackgroundPixmap, quitPixmap,
        XtNlabel, " ",
#else
        XtNlabel, "Quit",
#endif
        XtNfromVert, w_hardcopy,
        XtNvertDistance, 5,
        XtNhorizDistance, 5,
        XtNwidth, 42,
        XtNheight, 42,
#endif
        NULL );
    XtAddCallback( w_quit,
#ifndef USE_ATHENA
        XmNactivateCallback, (XtCallbackProc)CanvasQuit,
#else
        XtNcallback, (XtCallbackProc)CanvasQuit,
#endif
        (XtPointer)NULL );

#ifdef XPM
    attributes.valuemask = XpmReturnPixels;
    status = XpmReadFileToPixmap( display,
        DefaultRootWindow(XtDisplay(toplevel)),
        "pixmaps/pex.xpm", &pexPixmap, &pix, &attributes );
    errorMessageXPM( status, "Pex" );
#endif

    if( status != 0 /* XpmSuccess */ ) {
        #include "pixmaps/pex.xbm"
        pexBitmap = XCreatePixmapFromBitmapData( display,
            DefaultRootWindow( display ),
            pex_bits, pex_width, pex_height,
            BlackPixel( display, DefaultScreen(display) ),
            WhitePixel( display, DefaultScreen(display) ), 8 );
        pexPixmap = pexBitmap;
    }

    w_pex = XtVaCreateManagedWidget( "w_pex",
#ifndef USE_ATHENA
        xmPushButtonWidgetClass, w_form1,
        XmNlabelString, XmStringCreateSimple( " " ),
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 10,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 10,
#ifdef XPM
        XmNlabelType, XmPIXMAP,
        XmNlabelPixmap, pexPixmap,
        XmNbackgroundPixmap, pexPixmap,
#endif
        XmNshadowThickness, 0,
        XmNhighlightThickness, 0,
        XmNwidth, 42,
        XmNheight, 42,
#else
        commandWidgetClass, w_form1,
        XtNlabel, " ",
        XtNfromVert, w_hardcopy,
        XtNvertDistance, 5,
        XtNfromHoriz, w_quit,
        XtNhorizDistance, 180,
#ifdef XPM
        XtNbackgroundPixmap, pexPixmap,
#endif
        XtNhighlightThickness, 0,
        XtNborderWidth, 0,
        XtNwidth, 42,
        XtNheight, 42,
#endif
        NULL );

    XtRealizeWidget( w_control );
    XtPopup( w_control, XtGrabNone );

#ifndef USE_ATHENA
    wm_delete_window = XmInternAtom( display, "WM_DELETE_WINDOW", False );
    XmAddWMProtocolCallback( w_control, wm_delete_window,
        (XtCallbackProc)quit, NULL );
#endif
}
